function [ln]=sphere_dist_der(p,q)

%computes the derivatives of the squared distance function
%d^2(f,g)=arccos^2(f'*g)

% input:    p, q are matrices. The squared distance function is 
%           computed for several pairs (p(i,:),q(i,:)) of vectors.
% output:   l{1,1} value of the squared distance function
%           l{2,1} first derivative of first argument
%           l{1,2} first derivative of second argument
%           l{3,1} derivative of order 2 of first argument
%           l{2,2} derivative of order 2 of first and second argument
%           l{1,3} derivative of order 2 of second argument

%For the computation an extension of this function is considered.
%If applied to tangent vectors the result are the intrinsic derivatives


[~,d]=size(p);
if d==1
    p=p';
    q=q';
    [~,d]=size(p);
end
x=tprod(p,[1 -1],q,[1 -1]);
[a,b]=beta(x);


%innerproduct derivatives
l1=lin_scalar_product_der(p,q);

%chain rule to compute extrinsic derivatives
l=chain21(l1,a);
ln=cell(3);


%intrinsic derivatives
ln{1,1}=l{1,1};
ln{2,1}=l{2,1};
ln{1,2}=l{1,2};
temp=tprod(b{1},[1 2],eye(d),[3 4]);
ln{3,1}=l{3,1}-temp;
ln{2,2}=l{2,2};
ln{1,3}=l{1,3}-temp;



for i=1:3
    for j=1:4-i
        ln{i,j}=squeeze(ln{i,j});
    end
end

end


function [res]=alpha(x)
arccos=acos(x);
onemx2=1-x.^2;
        %Falls |1-|x|| genuegend gross
        r1=arccos.^2;
        r2=-2*arccos./sqrt(onemx2);
        r3=(2+x.*r2)./onemx2;
        %r4=(6*x+(2*x.^2+1).*r2)./onemx2.^2;
        isn=isnan(r2);
        
        
        %Falls |1-|x|| sehr klein
        xm=abs(x)-1;
        xm2=xm.^2;
        xm3=xm.^3;
        z2=-2+2/3*xm-4/15*xm2+4/35*xm3;
        z3=2/3-8/15*xm+12/35*xm2;
        %z4=-8/15+24/35*xm;
        
        %Nan mit z werten ersetzen ersetzen
        r2(isn)=z2(isn);
        r3(isn)=z3(isn);
        %r4(isn)=z4(isn);
        
        %Als Zellenarray aufschreiben
        res=cell(2,1);%res=cell(4,1);
        res{1}=r1;
        res{2}=r2;
        res{3}=r3;
        %res{4}=r4;
        
end


function [a,b]=beta(x)
a=alpha(x);
b=cell(1,2);%b=cell(1,3);       
            b{1}=a{2}.*x;  
            b{2}=a{3}.*x+a{2};
            %b{3}=a{4}.*x+2*a{3};
end

